/**
 * \addtogroup Servo
 * @{
 */

/*****************************************************************************/
/**
 * \file Servo.c
 * Positive Pulse PWM Servo routines for 8051 and Keil C compiler.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 28-01-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "Servo.h"

/*****************************************************************************
 *
 *                              Global Variables
 *
 *****************************************************************************/
#ifdef SERVO_0_ENABLE
    unsigned char servo_0_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_1_ENABLE
    unsigned char servo_1_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_2_ENABLE
    unsigned char servo_2_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_3_ENABLE
    unsigned char servo_3_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_4_ENABLE
    unsigned char servo_4_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_5_ENABLE
    unsigned char servo_5_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_6_ENABLE
    unsigned char servo_6_posn = SERVO_POSN_MID;
#endif
#ifdef SERVO_7_ENABLE
    unsigned char servo_7_posn = SERVO_POSN_MID;
#endif


/*****************************************************************************
 *
 *                         Private Function Prototypes
 *
 *****************************************************************************/
//void timer2_isr(void) interrupt 3 using 2;
unsigned int get_phase_0_time(unsigned char servo_posn);
void set_timer2(unsigned int phase_time);


/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                               servo_init()              
 *
 *****************************************************************************/
void servo_init(void)
{
    SERVO_PORT = 0;       // Clear all servo outputs.
    TMOD &= 0x0F;         // Clear upper nibble of timer control.
    TMOD |= 0x10;         // Timer 1 mode 1 not gated, internal clock, 16 bit,
                          // no auto reload.
    TH1 = 0x00;           // Load Timer 1 high byte.  
    TL1 = 0x00;           // Load Timer 1 low byte.
                          // Interrupt occurs when timer overflows on up count.
    TR1 = 1;              // Start Timer 1. TR1 is run control bit in TCON.
    ET1 = 1;              // Enable Timer 1 interrupt.
    EA  = 1;              // Enable Global interrupt. 
}


/*****************************************************************************
 *
 *                        Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                                timer2_isr()  
 *
 *****************************************************************************/
void timer2_isr(void) interrupt 3 using 2
{
    static unsigned char servo = SERVO_0;   // Start with 1st servo
    static bit phase = 0;                   // Start with phase 0 of servo 
                                            // output cycle
    static unsigned int phase_0_time;       // Calculate phase_0 and phase_1 
                                            // time in one interrupt service 
                                            // and hold 2nd result as servo 
                                            // posn can change between 
                                            // invocations giving erratic 
                                            // timing
    switch (servo)
    {
        case SERVO_0:
        {
#ifdef SERVO_0_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_0_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_0_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_0_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_1;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_1;
#endif
            break;
        }
        case SERVO_1:
        {
#ifdef SERVO_1_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_1_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_1_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_1_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_2;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_2;
#endif
            break;
        }
        case SERVO_2:
        {
#ifdef SERVO_2_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_2_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_2_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_2_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_3;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_3;
#endif
            break;
        }
        case SERVO_3:
        {
#ifdef SERVO_3_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_3_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_3_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_3_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_4;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_4;
#endif
            break;
        }
        case SERVO_4:
        {
#ifdef SERVO_4_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_4_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_4_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_4_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_5;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_5;
#endif
            break;
        }
        case SERVO_5:
        {
#ifdef SERVO_5_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_5_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_5_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_5_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_6;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_6;
#endif
            break;
        }
        case SERVO_6:
        {
#ifdef SERVO_6_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_6_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_6_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_6_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_7;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_7;
#endif
            break;
        }
        case SERVO_7:
        {
#ifdef SERVO_7_ENABLE
            if (phase == 0)
            {
                phase_0_time = get_phase_0_time(servo_7_posn);
                set_timer2(CYCLES_FOR_OVERFLOW - phase_0_time);
                SERVO_7_OUT = 1;
                phase = 1;
            }
            else // phase = 1
            {
                SERVO_7_OUT = 0;
                set_timer2(CYCLES_FOR_OVERFLOW - (CYCLES_FOR_2MS5
                                                        - phase_0_time));
                servo = SERVO_0;
                phase = 0;
            }
#else
            set_timer2(CYCLES_FOR_OVERFLOW - CYCLES_FOR_2MS5);
            servo = SERVO_0;
#endif
            break;
        }
    }
}

/*****************************************************************************
 *
 *                              get_phase_0_time()  
 *
 *****************************************************************************/
unsigned int get_phase_0_time(unsigned char servo_posn) using 2
{
    return (unsigned int)(CYCLES_FOR_1MS + ((CYCLES_FOR_2MS - CYCLES_FOR_1MS) 
                                          / SERVO_POSN_MAX * servo_posn));
}

/*****************************************************************************
 *
 *                                set_timer2()  
 *
 *****************************************************************************/
void set_timer2(unsigned int phase_time) using 2
{
    TR1 = 0;                       // Stop Timer 1 
    TH1 = phase_time >> 8;         // Reload Timer 1 high byte.  
    TL1 = phase_time & 0xFF;       // Reload Timer 1 low byte.
    TR1 = 1;                       // Restart Timer 1 
}

/** @} */
